// CursorShielder.m - when the cursor isn't moving, hide it,
//
//  Created by David Phillip Oster on 4/1/09.
//  Copyright 2009 David Phillip Oster. All rights reserved.
//

// Tests: verfiied that simply writing mouseMoved handlers or just calling 
// [NSCursor setHiddenUntilMouseMoves:YES]; did not work.
// Algorithm:
// * Manage a TrackingRect the size of the owning view's bounds
//  (tracking changes)
// * When the cursor enters TrackingRect, start a repeating timer. Stop timer
//   when cursor leaves.
// * When timer fires, call [NSCursor setHiddenUntilMouseMoves:YES], which hides
//   the cursor until the mouose moves next.

#import "CursorShielder.h"

@interface CursorShielder(Private)
- (void)setView:(NSView *)view;
- (void)setHiddenUntilMouseMoves:(BOOL)doHide;
@end

@implementation CursorShielder

- (id)initWithView:(NSView *)view {
  self = [super init];
  if (self) {
    [self setView:view];
  }
  return self;
}

- (void)dealloc {
  [self setHiddenUntilMouseMoves:NO];
  [self setView:nil];
  [super dealloc];
}

- (void)invalidate {
  [self setView:nil];
}

- (void)resetShieldFrame:(NSNotification *)notify {
  if (view_ && 0 != trackingRectTag_) {
    [view_ removeTrackingRect:trackingRectTag_];
  }
  NSRect r = [view_ bounds];
  trackingRectTag_ = [view_ addTrackingRect:r owner:self userData:view_ assumeInside:NO];
  NSPoint windowPoint = [[view_ window] convertScreenToBase:[NSEvent mouseLocation]];
  BOOL isInside = NSPointInRect(windowPoint, [view_ frame]);
  if (isInside_ != isInside) {
    [self setHiddenUntilMouseMoves:isInside];
    isInside_ = isInside;
  }
}

- (void)mouseEntered:(NSEvent *)event {
  if (view_ == [event userData]) {
    isInside_ = YES;
    [self setHiddenUntilMouseMoves:YES];
  }
}


- (void)mouseExited:(NSEvent *)event {
  if (view_ == [event userData]) {
    isInside_ = NO;
    [self setHiddenUntilMouseMoves:NO];
  }
}

- (void)setHiddenUntilMouseMoves:(BOOL)doHide {
  [mouseCheckTimer_ invalidate];
  [mouseCheckTimer_ release];
  mouseCheckTimer_ = nil;
  if (doHide) {
    mouseCheckTimer_ = 
      [[NSTimer scheduledTimerWithTimeInterval:0.5
                                        target:self 
                                      selector:@selector(mouseCheckTimer:) 
                                      userInfo:nil 
                                      repeats:YES] retain];
  }
}

- (void)mouseCheckTimer:(NSTimer *)timer {
  if ([[view_ window] isMainWindow] && isInside_) {
    [NSCursor setHiddenUntilMouseMoves:YES];
  }
}



- (void)setView:(NSView *)view {
  NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
  [nc removeObserver:self];
  view_ = view;
  if (view_) {
    [nc addObserver:self selector:@selector(resetShieldFrame:) 
      name:NSViewFrameDidChangeNotification object:view];
    [nc addObserver:self selector:@selector(resetShieldFrame:) 
      name:NSViewBoundsDidChangeNotification object:view];
    [nc addObserver:self selector:@selector(resetShieldFrame:) 
      name:NSViewGlobalFrameDidChangeNotification object:view];
    [self resetShieldFrame:nil];
  }
}

@end

